@@ -1,4 +1,4 @@
-# This file is part of CycloneDX Python Lib
+# This file is part of CycloneDX Python Library
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ a schema extension: https://cyclonedx.org/ext/vulnerability.
 Since CycloneDX schema version 1.4, this has become part of the core schema.
 
 .. note::
-    See the CycloneDX Schema extension definition https://cyclonedx.org/docs/1.4/#type_vulnerabilitiesType
+    See the CycloneDX Schema extension definition https://cyclonedx.org/docs/1.6/#type_vulnerabilitiesType
 """
 
 
@@ -35,14 +35,14 @@ from decimal import Decimal
 from enum import Enum
 from typing import Any, Dict, FrozenSet, Iterable, Optional, Tuple, Type, Union
 
-import serializable
+import py_serializable as serializable
 from sortedcontainers import SortedSet
 
+from .._internal.bom_ref import bom_ref_from_str as _bom_ref_from_str
 from .._internal.compare import ComparableTuple as _ComparableTuple
 from ..exception.model import MutuallyExclusivePropertiesException, NoPropertiesProvidedException
 from ..schema.schema import SchemaVersion1Dot4, SchemaVersion1Dot5, SchemaVersion1Dot6
-from ..serialization import BomRefHelper
-from . import Property, Tool, XsUri
+from . import Property, XsUri
 from .bom_ref import BomRef
 from .contact import OrganizationalContact, OrganizationalEntity
 from .impact_analysis import (
@@ -51,6 +51,7 @@ from .impact_analysis import (
     ImpactAnalysisResponse,
     ImpactAnalysisState,
 )
+from .tool import Tool, ToolRepository, _ToolRepositoryHelper
 
 
 @serializable.serializable_class
@@ -61,7 +62,7 @@ class BomTargetVersionRange:
     `version` and `version_range` are mutually exclusive.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -84,6 +85,7 @@ class BomTargetVersionRange:
 
     @property
     @serializable.xml_sequence(1)
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
     def version(self) -> Optional[str]:
         """
         A single version of a component or service.
@@ -123,22 +125,23 @@ class BomTargetVersionRange:
     def status(self, status: Optional[ImpactAnalysisAffectedStatus]) -> None:
         self._status = status
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.version, self.range, self.status
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, BomTargetVersionRange):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, BomTargetVersionRange):
-            return _ComparableTuple((
-                self.version, self.range, self.status
-            )) < _ComparableTuple((
-                other.version, other.range, other.status
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.version, self.range, self.status))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<BomTargetVersionRange version={self.version}, version_range={self.range}, status={self.status}>'
@@ -155,7 +158,7 @@ class BomTarget:
     from a `cyclonedx.parser.BaseParser` implementation.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_vulnerabilityType
     """
 
     def __init__(
@@ -194,18 +197,24 @@ class BomTarget:
     def versions(self, versions: Iterable[BomTargetVersionRange]) -> None:
         self._versions = SortedSet(versions)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.ref,
+            _ComparableTuple(self.versions)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, BomTarget):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, BomTarget):
-            return self.ref < other.ref
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.ref, tuple(self.versions)))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<BomTarget ref={self.ref}>'
@@ -217,7 +226,7 @@ class VulnerabilityAnalysis:
     Class that models the `analysis` sub-element of the `vulnerabilityType` complex type.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -227,11 +236,6 @@ class VulnerabilityAnalysis:
         responses: Optional[Iterable[ImpactAnalysisResponse]] = None,
         detail: Optional[str] = None,
     ) -> None:
-        if not state and not justification and not responses and not detail:
-            raise NoPropertiesProvidedException(
-                'At least one of state, justification, responses or detail must be provided for VulnerabilityAnalysis '
-                '- none supplied'
-            )
         self.state = state
         self.justification = justification
         self.responses = responses or []  # type:ignore[assignment]
@@ -323,13 +327,18 @@ class VulnerabilityAnalysis:
     # def last_updated(self, ...) -> None:
     #     ...  # TODO since CDX 1.5
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.state, self.justification, tuple(self.responses), self.detail
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityAnalysis):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __hash__(self) -> int:
-        return hash((self.state, self.justification, tuple(self.responses), self.detail))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAnalysis state={self.state}, justification={self.justification}>'
@@ -341,7 +350,7 @@ class VulnerabilityAdvisory:
     Class that models the `advisoryType` complex type.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_advisoryType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_advisoryType
     """
 
     def __init__(
@@ -354,6 +363,7 @@ class VulnerabilityAdvisory:
 
     @property
     @serializable.xml_sequence(1)
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
     def title(self) -> Optional[str]:
         """
         The title of this advisory.
@@ -376,22 +386,23 @@ class VulnerabilityAdvisory:
     def url(self, url: XsUri) -> None:
         self._url = url
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.title, self.url
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityAdvisory):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityAdvisory):
-            return _ComparableTuple((
-                self.title, self.url
-            )) < _ComparableTuple((
-                other.title, other.url
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.title, self.url))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAdvisory url={self.url}, title={self.title}>'
@@ -405,7 +416,7 @@ class VulnerabilitySource:
     This type is used for multiple purposes in the CycloneDX schema.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilitySourceType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilitySourceType
     """
 
     def __init__(
@@ -413,15 +424,12 @@ class VulnerabilitySource:
         name: Optional[str] = None,
         url: Optional[XsUri] = None,
     ) -> None:
-        if not name and not url:
-            raise NoPropertiesProvidedException(
-                'Either name or url must be provided for a VulnerabilitySource - neither provided'
-            )
         self.name = name
         self.url = url
 
     @property
     @serializable.xml_sequence(1)
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
     def name(self) -> Optional[str]:
         """
         Name of this Source.
@@ -444,22 +452,23 @@ class VulnerabilitySource:
     def url(self, url: Optional[XsUri]) -> None:
         self._url = url
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.name, self.url
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilitySource):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilitySource):
-            return _ComparableTuple((
-                self.name, self.url
-            )) < _ComparableTuple((
-                other.name, other.url
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.name, self.url))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAdvisory name={self.name}, url={self.url}>'
@@ -476,61 +485,75 @@ class VulnerabilityReference:
     intelligence.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
+
+    .. note::
+        Properties ``id`` and ``source`` are mandatory.
+
+        History:
+        * In v1.4 JSON scheme, both properties were mandatory
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.4.schema.json#L1455-L1474
+        * In v1.4 XML schema, both properties were optional
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.4.xsd#L1788-L1797
+        * In v1.5 XML schema, both were mandatory
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.5.xsd#L3364-L3374
+
+        Decision:
+        Since CycloneDXCoreWorkingGroup chose JSON schema as the dominant schema, the one that serves as first spec
+        implementation, and since XML schema was "fixed" to work same as JSON schema, we'd consider it canon/spec that
+        both properties were always mandatory.
     """
 
     def __init__(
         self, *,
-        id: Optional[str] = None,
-        source: Optional[VulnerabilitySource] = None,
+        id: str,
+        source: VulnerabilitySource,
     ) -> None:
-        if not id and not source:
-            raise NoPropertiesProvidedException(
-                'Either id or source must be provided for a VulnerabilityReference - neither provided'
-            )
         self.id = id
         self.source = source
 
     @property
     @serializable.xml_sequence(1)
-    def id(self) -> Optional[str]:
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
+    def id(self) -> str:
         """
         The identifier that uniquely identifies the vulnerability in the associated Source. For example: CVE-2021-39182.
         """
         return self._id
 
     @id.setter
-    def id(self, id: Optional[str]) -> None:
+    def id(self, id: str) -> None:
         self._id = id
 
     @property
     @serializable.xml_sequence(2)
-    def source(self) -> Optional[VulnerabilitySource]:
+    def source(self) -> VulnerabilitySource:
         """
         The source that published the vulnerability.
         """
         return self._source
 
     @source.setter
-    def source(self, source: Optional[VulnerabilitySource]) -> None:
+    def source(self, source: VulnerabilitySource) -> None:
         self._source = source
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.id, self.source
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityReference):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityReference):
-            return _ComparableTuple((
-                self.id, self.source
-            )) < _ComparableTuple((
-                other.id, other.source
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.id, self.source))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityReference id={self.id}, source={self.source}>'
@@ -542,7 +565,7 @@ class VulnerabilityScoreSource(str, Enum):
     Enum object that defines the permissible source types for a Vulnerability's score.
 
     .. note::
-        See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/#type_scoreSourceType
+        See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.6/#type_scoreSourceType
 
     .. note::
         No explicit carry-over from the former schema extension:
@@ -664,7 +687,7 @@ class VulnerabilitySeverity(str, Enum):
     Class that defines the permissible severities for a Vulnerability.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_severityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_severityType
     """
     NONE = 'none'
     INFO = 'info'  # Only >= 1.4
@@ -718,7 +741,7 @@ class VulnerabilityRating:
     1.4 - see https://github.com/CycloneDX/specification/blob/master/schema/ext/vulnerability-1.0.xsd.
 
     .. note::
-        See `ratingType` in https://cyclonedx.org/docs/1.4/#ratingType
+        See `ratingType` in https://cyclonedx.org/docs/1.6/xml/#ratingType
 
     .. warning::
         As part of implementing support for CycloneDX schema version 1.4, the three score types defined in the schema
@@ -736,11 +759,6 @@ class VulnerabilityRating:
         vector: Optional[str] = None,
         justification: Optional[str] = None,
     ) -> None:
-        if not source and not score and not severity and not method and not vector and not justification:
-            raise NoPropertiesProvidedException(
-                'At least one property must be provided when creating a VulnerabilityRating - none supplied.'
-            )
-
         self.source = source
         self.score = score
         self.severity = severity
@@ -803,6 +821,7 @@ class VulnerabilityRating:
 
     @property
     @serializable.xml_sequence(5)
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
     def vector(self) -> Optional[str]:
         """
         The textual representation of the metric values used to score the vulnerability - also known as the vector.
@@ -825,25 +844,30 @@ class VulnerabilityRating:
     def justification(self, justification: Optional[str]) -> None:
         self._justification = justification
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.severity, self.score or 0,
+            self.source, self.method, self.vector,
+            self.justification
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityRating):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityRating):
-            return _ComparableTuple((
-                self.severity, self.score, self.score or 0, self.method, self.vector, self.justification
-            )) < _ComparableTuple((
-                other.severity, other.score, other.score or 0, other.method, other.vector, other.justification
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.source, self.score or 0, self.severity, self.method, self.vector, self.justification))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
-        return f'<VulnerabilityRating score={self.score}, source={self.source}>'
+        return f'<VulnerabilityRating severity={self.severity} score={self.score}, ' \
+            f'source={self.source} method={self.method} vector={self.vector}' \
+            f'justification={self.justification}>'
 
 
 @serializable.serializable_class
@@ -855,7 +879,7 @@ class VulnerabilityCredits:
     extension (in XML only).
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -863,10 +887,6 @@ class VulnerabilityCredits:
         organizations: Optional[Iterable[OrganizationalEntity]] = None,
         individuals: Optional[Iterable[OrganizationalContact]] = None,
     ) -> None:
-        if not organizations and not individuals:
-            raise NoPropertiesProvidedException(
-                'One of `organizations` or `individuals` must be populated - neither were'
-            )
         self.organizations = organizations or []  # type:ignore[assignment]
         self.individuals = individuals or []  # type:ignore[assignment]
 
@@ -902,18 +922,24 @@ class VulnerabilityCredits:
     def individuals(self, individuals: Iterable[OrganizationalContact]) -> None:
         self._individuals = SortedSet(individuals)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            _ComparableTuple(self.organizations),
+            _ComparableTuple(self.individuals)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityCredits):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityCredits):
-            return hash(self) < hash(other)
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((tuple(self.organizations), tuple(self.individuals)))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityCredits id={id(self)}>'
@@ -928,7 +954,7 @@ class Vulnerability:
     extension (in XML only).
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_vulnerabilityType
     """
 
     def __init__(
@@ -948,15 +974,12 @@ class Vulnerability:
         published: Optional[datetime] = None,
         updated: Optional[datetime] = None,
         credits: Optional[VulnerabilityCredits] = None,
-        tools: Optional[Iterable[Tool]] = None,
+        tools: Optional[Union[Iterable[Tool], ToolRepository]] = None,
         analysis: Optional[VulnerabilityAnalysis] = None,
         affects: Optional[Iterable[BomTarget]] = None,
         properties: Optional[Iterable[Property]] = None,
     ) -> None:
-        if isinstance(bom_ref, BomRef):
-            self._bom_ref = bom_ref
-        else:
-            self._bom_ref = BomRef(value=str(bom_ref) if bom_ref else None)
+        self._bom_ref = _bom_ref_from_str(bom_ref)
         self.id = id
         self.source = source
         self.references = references or []  # type:ignore[assignment]
@@ -978,15 +1001,13 @@ class Vulnerability:
 
     @property
     @serializable.json_name('bom-ref')
-    @serializable.type_mapping(BomRefHelper)
+    @serializable.type_mapping(BomRef)
     @serializable.xml_attribute()
     @serializable.xml_name('bom-ref')
     def bom_ref(self) -> BomRef:
         """
         Get the unique reference for this Vulnerability in this BOM.
 
-        If a value was not provided in the constructor, a UUIDv4 will have been assigned.
-
         Returns:
            `BomRef`
         """
@@ -994,6 +1015,7 @@ class Vulnerability:
 
     @property
     @serializable.xml_sequence(1)
+    @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
     def id(self) -> Optional[str]:
         """
         The identifier that uniquely identifies the vulnerability. For example: CVE-2021-39182.
@@ -1240,20 +1262,22 @@ class Vulnerability:
         self._credits = credits
 
     @property
-    @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'tool')
+    @serializable.type_mapping(_ToolRepositoryHelper)
     @serializable.xml_sequence(17)
-    def tools(self) -> 'SortedSet[Tool]':
+    def tools(self) -> ToolRepository:
         """
-        The tool(s) used to identify, confirm, or score the vulnerability.
+        Tools used to create this BOM.
 
         Returns:
-            Set of `Tool`
+            :class:`ToolRepository` object.
         """
         return self._tools
 
     @tools.setter
-    def tools(self, tools: Iterable[Tool]) -> None:
-        self._tools = SortedSet(tools)
+    def tools(self, tools: Union[Iterable[Tool], ToolRepository]) -> None:
+        self._tools = tools \
+            if isinstance(tools, ToolRepository) \
+            else ToolRepository(tools=tools)
 
     @property
     @serializable.xml_sequence(18)
@@ -1303,26 +1327,30 @@ class Vulnerability:
     def properties(self, properties: Iterable[Property]) -> None:
         self._properties = SortedSet(properties)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.id, self.bom_ref.value,
+            self.source, _ComparableTuple(self.references),
+            _ComparableTuple(self.ratings), _ComparableTuple(self.cwes), self.description,
+            self.detail, self.recommendation, self.workaround, _ComparableTuple(self.advisories),
+            self.created, self.published, self.updated,
+            self.credits, self.tools, self.analysis,
+            _ComparableTuple(self.affects),
+            _ComparableTuple(self.properties)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Vulnerability):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, Vulnerability):
-            return _ComparableTuple((
-                self.id, self.description, self.detail, self.source, self.created, self.published
-            )) < _ComparableTuple((
-                other.id, other.description, other.detail, other.source, other.created, other.published
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((
-            self.id, self.source, tuple(self.references), tuple(self.ratings), tuple(self.cwes), self.description,
-            self.detail, self.recommendation, self.workaround, tuple(self.advisories), self.created, self.published,
-            self.updated, self.credits, tuple(self.tools), self.analysis, tuple(self.affects), tuple(self.properties)
-        ))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<Vulnerability bom-ref={self.bom_ref.value}, id={self.id}>'
